;;; COPYRIGHT 2006 Sergio Garcia <sergio.garcia@gmail.com>
;;; 
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU Lesser General Public License as published by
;;; the Free Software Foundation; either version 2.1 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



(in-package #:gordon)

(defstruct movie
  (objects-list '())
  (xsize 100 :type (integer * *))
  (ysize 100 :type (integer * *))
  (frame-rate 10 :type (integer * *))
  )



;;; Marshall a movie

(defun marshall-movie (movie)
  (let* (
	 (marshalled-tags (apply #'xappend (mapcar #'marshall-movie-object (movie-objects-list movie))))
	 (flat-marshalled-tags (flat-marshall-data marshalled-tags))
	 (tags-length (length flat-marshalled-tags))
	 (marshalled-movie-size (marshall-rect (make-rect 
						:xmax (movie-xsize movie)
						:ymax (movie-ysize movie))))
	 (flat-marshalled-movie-size (flat-marshall-data marshalled-movie-size))
	 (file-size (+ 12
		       (length flat-marshalled-movie-size)
		       tags-length)))
    (xappend
     (i2ui8 (char-code #\F)) ;F
     (i2ui8 (char-code #\W)) ;W
     (i2ui8 (char-code #\S)) ;S 
     (i2ui8 *flash-version*)  ; version 
     (i2ui32 file-size)  ; length
     flat-marshalled-movie-size
     (i2ui8 0) ; this byte is ignored
     (i2ui8 (movie-frame-rate movie)) ; rate
     (i2ui16 (movie-frames movie)) ; number of frames
     flat-marshalled-tags)))

(defun write-movie (movie file)  
  (with-open-file (outputstream file 
				:direction :output 
				:if-exists :supersede
				:element-type '(unsigned-byte 8))
				
    (write-sequence (flat-marshall-data (marshall-movie movie)) outputstream)))

(defun add-movie-object (movie object)
   (setf (movie-objects-list movie) (append (movie-objects-list movie) (list object))))

(defun add-tag (movie tag)
  (setf (movie-objects-list movie) (append (movie-objects-list movie) (list tag))))


(defun add-shape (m id depth shape x y) 
  (add-tag m (make-tag-define-shape 
	      :id id
	      :shape shape))

  (add-tag m (make-tag-place-object2 
	      :depth depth
	      :id id
	      :matrix (make-matrix :translate-x x
				   :translate-y y))))




(defun movie-frames (m)
  (count 1 (mapcar #'tag-type (movie-objects-list m))))

(defmacro with-movie 
    ((varname filename &key (frame-rate 12) (width 10000) (height 10000) (bgcolor nil)) &body body)
  `(let ((,varname 
	  (make-movie :xsize ,width 
		      :ysize ,height 
		      :frame-rate ,frame-rate)))
    ,@(if bgcolor `((add-tag ,varname (make-tag-SBC :bgcolor ,bgcolor))))
    ,@body
    (write-movie ,varname ,filename)))



