;;; COPYRIGHT 2006 Sergio Garcia <sergio.garcia@gmail.com>
;;; 
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU Lesser General Public License as published by
;;; the Free Software Foundation; either version 2.1 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



(defparameter *tag-data-marshallers* (make-hash-table))
(defparameter *tag-codes* (make-hash-table))

;;; TAGS marshalling framework
;;; we use a hashtable of lambdas instead of defmethod
;;; just to see.

(defun tag-type (tg)
  (gethash (type-of tg) *tag-codes*))

(defun marshall-tag-data (tg)
  (funcall (gethash (type-of tg) *tag-data-marshallers*) tg))

(defmacro add-tag-type(name code fields marshaller)
  `(progn (defstruct ,name ,@fields)
	 (setf (gethash ',name *tag-codes*) ,code)
	 (setf (gethash ',name *tag-data-marshallers*) ,marshaller)
	 ))

(defmacro add-tag-type(name code fields marshaller)
  `(progn (defstruct ,name ,@fields)
	 (setf (gethash ',name *tag-codes*) ,code)
	 (setf (gethash ',name *tag-data-marshallers*) ,marshaller)
	 ))

(defmacro macropepe (bindings &body rest)
  `(macrolet ((with-fields (&body rest2) `(let
					   ,,bindings 
					   ,@rest2)))
    ,@rest))

(pprint (macroexpand-1 '(macropepe '((a 1) (b 2)) (with-fields a))))
(macropepe '((a 1)) (with-fields a))
;;; Marshall a tag

(defun marshall-tag (tg)
  (let* ((mtd (flat-marshall-data (marshall-tag-data tg)))
	 (tt (tag-type tg))
	 (l (length mtd))
	 (long? (>= l 63))
	 (header-size (if long? 6 2))
	 (recordheader (if long? 
			   (list 
			    (is2leubitaba `(,tt 63) '(10 6))
			    (i2ui32 l))
			   (is2leubitaba `(,tt ,l) '(10 6)))))
    (xappend recordheader mtd)))

;;;
;;; Define all the different tag types
;;;


(add-tag-type tag-end
	      0
	      ()
	      (lambda (x) *empty-ba*))

(add-tag-type tag-showframe
	      1
	      ()
	      (lambda (x) *empty-ba*))


(add-tag-type tag-SBC 
	      9 
	      (bgcolor)
	      (lambda (x)
		(marshall-rgb (tag-SBC-bgcolor x))))


(add-tag-type tag-framelabel
	      43
	      (name)
	      (lambda (x) (string2string (tag-framelabel-name x))))


(add-tag-type tag-define-shape
	      32
	      (id shape)
	      (lambda (x) (xappend
			   (i2ui16 (tag-define-shape-id x))
			   (marshall-rect (calculate-shape-bounds (tag-define-shape-shape x)))
			   (marshall-shape-with-style (tag-define-shape-shape x)))))

(add-tag-type tag-place-object2
	      26
	      ((depth nil) (id nil) (move nil) (matrix nil) (cx nil) (name nil))
	      (lambda (x)
		(xappend 
		 *no-bit*
		 *no-bit*
		 (bif (tag-place-object2-name x))
		 *no-bit*
		 (bif (tag-place-object2-cx x))
		 (bif (tag-place-object2-matrix x))
		 (bif (tag-place-object2-id x))
		 (bif (tag-place-object2-move x))
		 (i2ui16 (tag-place-object2-depth x))
		 (when (tag-place-object2-id x) 
		   (i2ui16 (tag-place-object2-id x)))
		 (when (tag-place-object2-matrix x)
		   (marshall-matrix (tag-place-object2-matrix x)))
		 (when (tag-place-object2-cx x)
		   (marshall-cxform-wa (tag-place-object2-cx x)))
		 (when (tag-place-object2-name x)
		   (string2string (tag-place-object2-name x))))))

(add-tag-type tag-remove-object
	      28
	      ((depth nil))
	      (lambda (x) 
		(i2ui16 (tag-remove-object-depth x))))


	      

;(add-tag-type tag-define-font-2
;	      48
;	      ((id 0 :type integer)
;	       (name "stupid-font" :type string)
;	       (offsets nil) ;defined in defineFont
;	       (shapes nil) ;defined in defineFont
;	       (codes nil))
;	      (lambda (font)
;		(xappend
;		 (i2ui16 




